@@ -45,7 +45,7 @@ rollapply.ts <- function(data, ...) {
 
 rollapply.zoo <- function(data, width, FUN, ..., by = 1, 
 	by.column = TRUE, fill = if (na.pad) NA, na.pad = FALSE,
-	partial = FALSE, align = c("center", "left", "right")) {
+	partial = FALSE, align = c("center", "left", "right"), coredata = TRUE) {
 
 	if (!missing(na.pad)) {
 		warning("na.pad argument is deprecated")
@@ -69,7 +69,7 @@ rollapply.zoo <- function(data, width, FUN, ..., by = 1,
 			lapply(1:NCOL(data), function(j)
 				rollapply(data[, j, drop = TRUE], width = width, FUN = FUN, ...,
 					by = by, by.column = by.column, fill = fill,
-					partial = partial, align = align)
+					partial = partial, align = align, coredata = coredata)
 			)
 		)
 		if (NCOL(data) == 1) dim(z) <- c(length(z), 1)
@@ -83,16 +83,17 @@ rollapply.zoo <- function(data, width, FUN, ..., by = 1,
 	align <- match.arg(align)
 
 	if (!is.list(width)) width <- lapply(width, function(w) {
-			if (align == "right") seq(to = 0, length = w)
-			else if (align == "center") seq(to = floor(w/2), length = w)
-			else seq(from = 0, length = w)
+			if (align == "right") seq(to = 0, length.out = w)
+			else if (align == "center") seq(to = floor(w/2), length.out = w)
+			else seq(from = 0, length.out = w)
 	})
 	# recycle width (using by if length(width) == 1)
 	width <- if (length(width) == 1) {
 		w <- rep(list(NULL), NROW(data))
 		start.at <- if (partial < 0) max(-min(width[[1]]), 0) + 1 else 1
+		start.at <- min(NROW(data), start.at)
 		replace(w, seq(start.at, NROW(data), by = by), width)
-	} else rep(width, length = NROW(data))
+	} else rep(width, length.out = NROW(data))
 
 	f <- if (is.null(dim(data))) {
 		# undimensioned
@@ -125,8 +126,9 @@ rollapply.zoo <- function(data, width, FUN, ..., by = 1,
 		}
 	}
 
+        dat <- if(coredata) coredata(data) else data
 	dat <- mapply(f, seq_along(time(data)), width, 
-		MoreArgs = list(data = coredata(data), ...), SIMPLIFY = FALSE) 
+		MoreArgs = list(data = dat, ...), SIMPLIFY = FALSE) 
 		
 	ix <- !sapply(dat, is.null) # integer indexes of non-nulls
 
